# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class FindPetsByStatusRequestStatus(str, enum.Enum):
    AVAILABLE = "available"
    PENDING = "pending"
    SOLD = "sold"

    def visit(
        self,
        available: typing.Callable[[], T_Result],
        pending: typing.Callable[[], T_Result],
        sold: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is FindPetsByStatusRequestStatus.AVAILABLE:
            return available()
        if self is FindPetsByStatusRequestStatus.PENDING:
            return pending()
        if self is FindPetsByStatusRequestStatus.SOLD:
            return sold()
